﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using VehicleStatsBL.Interface;

namespace VehicleStatsBL.Impl
{
    public class Statistics : IStatistics
    {
        private IEnumerable<IVehicle> _vehicles;
        public Statistics(IEnumerable<IVehicle> vehicles)
        {
            _vehicles = vehicles;
        }

        public double MeanPrice
        {
            get { return _vehicles.Average(p => p.Price); }
        }

        public double MedianPrice
        {
            get
            {
                var orderedVehicles = _vehicles.OrderBy(p => p.Price);
                int totalNumber = _vehicles.Count();
                int position = totalNumber / 2;

                double median;
                if (totalNumber % 2 == 0)
                {
                    median = (orderedVehicles.ElementAt(position).Price + orderedVehicles.ElementAt(position - 1).Price) / 2;
                }
                else
                {
                    median = orderedVehicles.ElementAt(position).Price;
                }

                return median;
            }
        }

        public double ModePrice
        {
            get
            {
                return _vehicles.GroupBy(p => p.Price)
                    .OrderByDescending(group => group.Count())
                    .Select(p => p.Key)
                    .FirstOrDefault();

            }
        }

        public double StandardDeviation
        {
            get
            {
                double average = MeanPrice;
                double sumOfSquaresOfDifferences = _vehicles.Select(val => (val.Price - average) * (val.Price - average)).Sum();
                double sd = Math.Sqrt(sumOfSquaresOfDifferences / _vehicles.Count());
                return sd;
            }
        }


        public double LowestPrice
        {
            get { return _vehicles.OrderBy(p => p.Price).FirstOrDefault().Price; }
        }

        public double HighestPrice
        {
            get { return _vehicles.OrderByDescending(p => p.Price).FirstOrDefault().Price; }
        }


        public int SampleSize
        {
            get { return _vehicles.Count(); }
        }

        public IEnumerable<Tuple<int, double, int>> MeanPriceByYear
        {
            get
            {
                return _vehicles.GroupBy(vehicle => vehicle.Year)
                    .Select(group => new Tuple<int, double, int>(group.Key, group.Average(vehicle => vehicle.Price), group.Count()))
                    .OrderByDescending(t => t.Item1);
            }
        }

        public IEnumerable<Tuple<int, double, int>> DepreciationByYear
        {
            get 
            {
                var meanPricePerYear = MeanPriceByYear;

                IEnumerable<Tuple<int, double, double, int>> joinedMeanPricePerYear = meanPricePerYear.Join(meanPricePerYear, 
                    p => p.Item1, 
                    i => i.Item1+1, 
                    (p,i) => new Tuple<int, double, double, int>(p.Item1, p.Item2, i.Item2, p.Item3));

                var results = joinedMeanPricePerYear.Select(p => new Tuple<int, double, int>(p.Item1, -1 * CalculateDepreciation(p.Item3, p.Item2), p.Item4));
                return results;
            }
        }

        public IEnumerable<Tuple<int, double>> DepreciationByYearCumulative
        {
            get
            {
                var newVehicleMeanPrice = MeanPriceByYear.FirstOrDefault();
                return _vehicles.GroupBy(vehicle => vehicle.Year)
                    .OrderByDescending(p => p.Key)
                    .Select(group => new Tuple<int, double>(group.Key, CalculateDepreciation(newVehicleMeanPrice.Item2, group.Average(v => v.Price))));
            }
        }

        private double CalculateDepreciation(double year1, double year2)
        {
            return (year2 - year1) / year1 * 100;
        }
    }
}
